/****************************************************************************
** Meta object code from reading C++ file 'kcm.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../plymouth-kcm-6.5.91/src/kcm.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcm.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KCMPlymouthE_t {};
} // unnamed namespace

template <> constexpr inline auto KCMPlymouth::qt_create_metaobjectdata<qt_meta_tag_ZN11KCMPlymouthE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCMPlymouth",
        "0$Yv9pumSIIeI3szkov9mvqcZE1iM",
        "selectedPluginChanged",
        "",
        "selectedPluginIndexChanged",
        "busyChanged",
        "showSuccessMessage",
        "message",
        "showErrorMessage",
        "load",
        "save",
        "defaults",
        "reloadModel",
        "onEntryEvent",
        "KNSCore::Entry",
        "entry",
        "uninstall",
        "plugin",
        "themesModel",
        "QStandardItemModel*",
        "selectedPlugin",
        "selectedPluginIndex",
        "busy"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'selectedPluginChanged'
        QtMocHelpers::SignalData<void()>(2, 3, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectedPluginIndexChanged'
        QtMocHelpers::SignalData<void()>(4, 3, QMC::AccessPublic, QMetaType::Void),
        // Signal 'busyChanged'
        QtMocHelpers::SignalData<void()>(5, 3, QMC::AccessPublic, QMetaType::Void),
        // Signal 'showSuccessMessage'
        QtMocHelpers::SignalData<void(const QString &)>(6, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'showErrorMessage'
        QtMocHelpers::SignalData<void(const QString &)>(8, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'load'
        QtMocHelpers::SlotData<void()>(9, 3, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<void()>(10, 3, QMC::AccessPublic, QMetaType::Void),
        // Slot 'defaults'
        QtMocHelpers::SlotData<void()>(11, 3, QMC::AccessPublic, QMetaType::Void),
        // Method 'reloadModel'
        QtMocHelpers::MethodData<void()>(12, 3, QMC::AccessPublic, QMetaType::Void),
        // Method 'onEntryEvent'
        QtMocHelpers::MethodData<void(const KNSCore::Entry &)>(13, 3, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Method 'uninstall'
        QtMocHelpers::MethodData<void(const QString &)>(16, 3, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'themesModel'
        QtMocHelpers::PropertyData<QStandardItemModel*>(18, 0x80000000 | 19, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Constant),
        // property 'selectedPlugin'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'selectedPluginIndex'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags, 1),
        // property 'busy'
        QtMocHelpers::PropertyData<bool>(22, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    uint qt_metaObjectHashIndex = 1;
    return QtMocHelpers::metaObjectData<KCMPlymouth, qt_meta_tag_ZN11KCMPlymouthE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_metaObjectHashIndex);
}
Q_CONSTINIT const QMetaObject KCMPlymouth::staticMetaObject = { {
    QMetaObject::SuperData::link<KQuickConfigModule::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCMPlymouthE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCMPlymouthE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KCMPlymouthE_t>.metaTypes,
    nullptr
} };

void KCMPlymouth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCMPlymouth *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->selectedPluginChanged(); break;
        case 1: _t->selectedPluginIndexChanged(); break;
        case 2: _t->busyChanged(); break;
        case 3: _t->showSuccessMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->showErrorMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->load(); break;
        case 6: _t->save(); break;
        case 7: _t->defaults(); break;
        case 8: _t->reloadModel(); break;
        case 9: _t->onEntryEvent((*reinterpret_cast<std::add_pointer_t<KNSCore::Entry>>(_a[1]))); break;
        case 10: _t->uninstall((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCMPlymouth::*)()>(_a, &KCMPlymouth::selectedPluginChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPlymouth::*)()>(_a, &KCMPlymouth::selectedPluginIndexChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPlymouth::*)()>(_a, &KCMPlymouth::busyChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPlymouth::*)(const QString & )>(_a, &KCMPlymouth::showSuccessMessage, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCMPlymouth::*)(const QString & )>(_a, &KCMPlymouth::showErrorMessage, 4))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QStandardItemModel* >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStandardItemModel**>(_v) = _t->themesModel(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->selectedPlugin(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->selectedPluginIndex(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->busy(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setSelectedPlugin(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setBusy(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *KCMPlymouth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCMPlymouth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCMPlymouthE_t>.strings))
        return static_cast<void*>(this);
    return KQuickConfigModule::qt_metacast(_clname);
}

int KCMPlymouth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KQuickConfigModule::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void KCMPlymouth::selectedPluginChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KCMPlymouth::selectedPluginIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KCMPlymouth::busyChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void KCMPlymouth::showSuccessMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KCMPlymouth::showErrorMessage(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
